import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "components"
import "./view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма закрытого документа
 */
BasicView {
    id: root

    readonly property var card: Session.document.firstActiveCard

    contentItem: RowLayout {
        spacing: 24
        // Левая часть - список позиций
        GroupBox {
            Style.class: "positionsListBackground"
            Layout.fillHeight: true
            Layout.preferredWidth: 456
            // Информационные оповещения
            SingleNotification {
                anchors { left: parent.left; right: parent.right; top: parent.top }
                height: 40
                filter: Notification.Info
                onClicked: topPanel.open()
            }
            Item {
                width: 240
                height: 310
                anchors.centerIn: parent
                Rectangle {
                    id: thirdCircle
                    anchors.top: parent.top
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: 240
                    radius: width / 2
                    height:  width
                    opacity: 0
                    color: Style.parameter("color.green")
                    Behavior on opacity { OpacityAnimator { duration: 500 } }

                }
                Rectangle {
                    id: secondCircle
                    anchors.centerIn: thirdCircle
                    width: 200
                    radius: width / 2
                    height:  width
                    opacity: 0
                    color: Style.parameter("color.green")
                    Behavior on opacity {
                        SequentialAnimation {
                            OpacityAnimator { duration: 800 }
                            ScriptAction { script: thirdCircle.opacity = 0.15 }
                        }
                    }
                }
                Rectangle {
                    anchors.centerIn: secondCircle
                    width: 160
                    radius: width / 2
                    height:  width
                    color: Style.parameter("color.green")
                    Image {
                        anchors.centerIn: parent
                        source: Style.parameter("icon.checkClosed")
                        height: 62
                        fillMode: Image.PreserveAspectFit
                        Component.onCompleted: mainWindow.scaleImage(this)
                    }
                    Component.onCompleted: secondCircle.opacity = 0.30
                }
                BasicLabel {
                    anchors.top: thirdCircle.bottom
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.topMargin: 16
                    Style.class: "documentClosedTitle"
                    Tr { code: "documentCloseFormDocumentStatusLabel"; defaultMessage: "Чек закрыт" }
                }
            }
        }
        // Правая часть - элементы управления
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            // Верхняя панель кассы
            TopPanel {
                id: topPanel
                anchors { left: parent.left; top: parent.top; right: parent.right }
            }
            ColumnLayout {
                anchors { left: parent.left; bottom: parent.bottom; right: parent.right; top: topPanel.bottom }
                anchors { rightMargin: 24; bottomMargin: 24; topMargin: 24 }
                spacing: 0
                // Критические оповещения
                SingleNotification {
                    id: errorNotification
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Error | Notification.Mode
                    onClicked: topPanel.open()
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 0 }
                // Сумма чека/заголовок панели действий
                Item {
                    id: titleGroup
                    Layout.fillWidth: true
                    Layout.preferredHeight: 48
                    BasicLabel {
                        id: title
                        anchors.left: parent.left
                        anchors.bottom: parent.bottom
                        Style.class: "documentSubtotalTotalTitle"
                        Tr { code: "documentSubtotalChangeSum"; defaultMessage: "Сдача" }
                    }
                    BasicLabel {
                        id: totalValue
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        Style.class: "documentSubtotalTotalValue"
                        text: root.isSubtotal ? Utils.formatMoneyItem(Session.document.currentPayableSum)
                                              : Utils.formatMoneyItem(Session.document.changeSum)
                        horizontalAlignment: Text.AlignRight
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 24 }
                Item {
                    Layout.fillWidth: true
                    property int earnRowHeigth: earnRow.visible ? 30 : 0
                    property int spentRowHeight: spentRow.visible ? 30 : 0
                    Layout.preferredHeight: earnRowHeigth + spentRowHeight
                    ColumnLayout {
                        anchors.fill: parent
                        spacing: 0
                        RowLayout {
                            id: spentRow
                            visible: card !== undefined && !Utils.moneyIsZero(card.spentSum)
                            BasicLabel {
                                Style.class: "bonusOperationName"
                                Tr { code: "documentSubtotalSpentBonus"; defaultMessage: "Списано" }
                            }
                            Line {
                                Layout.alignment: Qt.AlignBottom
                                Layout.bottomMargin: 5
                                Layout.fillWidth: true
                                strokeStyle: ShapePath.DashLine
                                dashPattern: [2, 3]
                                color: Style.parameter("color.gray")
                            }
                            BasicLabel {
                                Layout.preferredHeight: 20
                                Style.class: "bonusOperationAmount"
                                text: card !== undefined ? Utils.formatMoneyItem(card.spentSum) : ""
                            }
                        }
                        RowLayout {
                            id: earnRow
                            visible: card !== undefined && !Utils.moneyIsZero(card.earnedSum)
                            BasicLabel {
                                Style.class: "bonusOperationName"
                                Tr { code: "documentSubtotalEarnBonus"; defaultMessage: "Начислено" }
                            }
                            Line {
                                Layout.alignment: Qt.AlignBottom
                                Layout.bottomMargin: 5
                                Layout.fillWidth: true
                                strokeStyle: ShapePath.DashLine
                                dashPattern: [2, 3]
                                color: Style.parameter("color.gray")
                            }
                            BasicLabel {
                                Layout.preferredHeight: 20
                                Style.class: "bonusOperationAmount"
                                text: card !== undefined ? Utils.formatMoneyItem(card.earnedSum) : ""
                            }
                        }
                    }
                }
                Item { Layout.preferredHeight: 10 }
                // Заголовок
                PaymentListHeader {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 24
                }
                Item { Layout.preferredHeight: 10 }
                // Оплаты
                PaymentList {
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                }
                Item { Layout.preferredHeight: 10 }
                // Панель действий
                ActionPanel {
                    id: actionPanel
                    Layout.fillWidth: true
                    Layout.preferredHeight: 380 + (Style.resolution.height - 768) * (0.166 * rows)
                    context: Context.DocumentClose
                    onClicked: (actionCode) => { handler.processAction(actionCode) }
                }
            }
        }
    }

    // Обработчик событий
    DocumentSubtotalHandler {
        id: handler
    }
    // Фильтр и коннектор для работы сканеров
    FormEventFilter {
        id: eventFilter
        onKeyboardData: (data, rawData, source) => handler.input(data, rawData, source)
        onCancel: handler.escCommand()
    }
}
