import QtQuick
import QtQuick.Window
import QtQuick.Controls
import QtQuick.VirtualKeyboard.Settings

import "components"
import "./view"

import Artix.Core
import Artix.Gui

/**
 * @brief Главное окно программы
 */
ApplicationWindow {
    id: mainWindow
    visible: true
    width: primaryScreen.geometry.width
    height: primaryScreen.geometry.height
    readonly property real scaleFactor: primaryScreen.geometry.height / Style.resolution.height
    readonly property bool multisampling: scaleFactor > 1
    function scaleImage(image) {
        let setSourceSize = () => {
            image.sourceSize = Qt.size(image.width * scaleFactor, image.height * scaleFactor)
        }
        scaleFactorChanged.connect(setSourceSize)
        setSourceSize()
    }

    Component.onCompleted: {
        let locales = Config.getStringList("Language:keyboardLayouts", "ru,en")
        let activeLocales = []
        for (let i = 0; i < locales.length; i++) {
            switch(locales[i]) {
                case "ru": activeLocales.push("ru_RU"); break;
                case "en": activeLocales.push("en_US"); break;
            }
        }
        VirtualKeyboardSettings.activeLocales = activeLocales
        requestActivate()
    }

    // Заполнитель свободной области. Пока это просто картинка.
    // В дальнейшем предполагается поместить сюда какой-то функционал
    Image {
        x: 0; y: 0; z: workArea.z - 1
        width: workArea.x
        visible: workArea.x > 10
        source: Style.parameter("icon.banner")
        height: mainWindow.height - bottomPanel.height
        Component.onCompleted: mainWindow.scaleImage(this)
    }

    // Рабочая область
    Control {
        id: workArea
        x: mainWindow.width - (contentItem ? contentItem.implicitWidth * contentItem.scale : mainWindow.width)
        y: 0

        onContentItemChanged: {
            contentItem?.onEventFilterChanged?.connect(() => updateEventFilter())
            contentItem?.onEventReceiverChanged?.connect(() => updateEventFilter())
            updateEventFilter()
        }

        // Обновить фильтр формы и ресивер ввода для текущего элемента
        function updateEventFilter() {
            let eventFilter = contentItem?.eventFilter
            let eventReceiver = contentItem?.eventReceiver
            eventFilterConnector.update(eventFilter, eventReceiver)
        }
    }

    // Нижняя информационная панель
    footer: BottomPanel {
        id: bottomPanel
        scale: mainWindow.scaleFactor
        transformOrigin: Item.BottomLeft
    }

    // Коннектим сигналы из C++ движка
    Connections {
        id: connections
        target: engine

        // Показать представление контекста
        function onShowContextView(item) {
            item.parent = workArea
            bottomPanel.visible = item.withBottomPanel || false
            workArea.contentItem = item
        }

        // Скрыть представление контекста
        function onCloseContextView() {
            workArea.contentItem = null
        }

        // Показать представление диалога
        function onShowDialog(item) {
            item.eventFilterConnector = eventFilterConnector
            item.open()
        }

        // Скрыть представление диалога
        function onCloseDialog(item) {
            item.close()

            // После закрытия диалога, возврашаем ввод текущему элементу рабочей области
            workArea.updateEventFilter()
        }
    }

    EventFilterConnector {
        id: eventFilterConnector

        // Обновление фильтра и ресивера для текущей открытой формы
        function update(eventFilter, eventReceiver) {
            clear()
            if (eventFilter)
                filter = eventFilter
            if (eventReceiver)
                receiver = eventReceiver
        }
    }
}
