import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../../components"
import Artix.Gui
import Artix.Core

/**
 * @brief Делегат товара в списке позиций
 */
PositionDelegate {
    id: root
    property bool showImages: false      ///< Показывать изображения товаров
    property string imagePlaceholderPath ///< Путь к изображению-заглушке
    property bool hasImage: image && showImages
    height: (selected ? contentItem.implicitHeight : giCommonContent.implicitHeight) + topPadding + bottomPadding
            + (object.hasConsultant ? 24 : 0)
    contentItem: ColumnLayout {
        spacing: 0
        RowLayout {
            visible: object.hasConsultant
            Layout.bottomMargin: 12
            Image {
                Layout.preferredWidth: 12
                Layout.preferredHeight: 14
                source: Style.parameter("icon.position.consultant")
                Component.onCompleted: mainWindow.scaleImage(this)
            }
            BasicLabel {
                Layout.fillWidth: true
                color: Style.parameter("color.info")
                text: consultantName ? consultantName : ""
                elide: Text.ElideRight
                wrapMode: Text.WordWrap
                maximumLineCount: 1
            }
        }
        RowLayout {
            id: giCommonContent
            spacing: 0
            Layout.alignment: Qt.AlignTop
            // TODO: Картинки пока считаем не работающим функционалом. Ждем дизайн для доработки
            Image {
                id: imageRef
                Layout.preferredWidth: 57
                Layout.preferredHeight: 57
                fillMode: Image.PreserveAspectFit
                visible: root.showImages && (root.hasImage || root.imagePlaceholderPath.length > 0)
                source: root.hasImage ? image : root.imagePlaceholderPath
                Component.onCompleted: mainWindow.scaleImage(this)
            }
            ColumnLayout {
                spacing: selected ? 12 : 8
                Behavior on spacing { SmoothedAnimation { velocity: 200 } }

                BasicLabel {
                    Layout.preferredWidth: 288
                    Layout.alignment: Qt.AlignTop
                    Layout.preferredHeight: selected ? 51 : 34
                    Style.class: "positionDelegateName"
                    text: object.name
                    elide: Text.ElideRight
                }
                RowLayout {
                    spacing: 0
                    BasicLabel {
                        Layout.alignment: Qt.AlignVCenter
                        Style.class: "positionDelegateQuant"
                        text: {
                            let quant = object.unitFlag ? Utils.weightStr(object.bquant) : object.bquant.toFixed(0)
                            return quant + " x " + Utils.formatMoneyItem(object.price)
                        }
                    }
                    BasicLabel {
                        Layout.alignment: Qt.AlignVCenter
                        Style.class: "positionDelegateDisc"
                        visible: !Utils.moneyIsZero(object.disc_abs + object.bonusImpactSum)
                        text: " - " + Utils.formatMoneyItem(object.impactSum)
                    }
                    Item { Layout.preferredWidth: 12 }
                    BasicLabel {
                        Layout.alignment: Qt.AlignVCenter
                        id: discRateLabel
                        property int discRate: Math.floor(object.impactPercent)
                        Style.class: "positionDelegateDiscRate"
                        elide: Text.ElideRight
                        padding: 8
                        topPadding: 1
                        bottomPadding: 3
                        visible: discRate > 0
                        Tr { code: "positionDiscountRate"; defaultMessage: "скидка %1%"; args: [discRateLabel.discRate] }
                    }
                }
            }
            ColumnLayout {
                Layout.alignment: Qt.AlignTop
                spacing: 8
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "positionDelegateSum"
                    text: Utils.formatMoneyItem(object.sumb)
                    horizontalAlignment: Text.AlignRight
                }
                ListView {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 24
                    layoutDirection: ListView.RightToLeft
                    orientation: ListView.Horizontal
                    spacing: 4
                    model: tags
                    delegate: Image {
                        width: 24
                        height: 24
                        fillMode: Image.PreserveAspectFit
                        source: Style.parameter(url, url)
                        Component.onCompleted: mainWindow.scaleImage(this)
                    }
                }
            }
        }
        ColumnLayout {
            id: giSelectedContent
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignHCenter
            opacity: selected ? 1 : 0
            spacing: 0
            Item { Layout.preferredHeight: 16; visible: discountName !== undefined }
            BasicLabel {
                Style.class: "positionDelegateDiscountTitle"
                Layout.fillWidth: true
                Layout.preferredHeight: 16
                visible: discountName !== undefined
                Tr { code: "positionDiscount"; defaultMessage: "Акция" }
            }
            Item { Layout.preferredHeight: 4; visible: discountName !== undefined }
            BasicLabel {
                Style.class: "positionDelegateDiscountName"
                Layout.fillWidth: true
                Layout.preferredHeight: 20
                elide: Text.ElideRight
                visible: discountName !== undefined
                text: discountName ? discountName : ""
            }
            Item { Layout.preferredHeight: 24 }
            BasicButton {
                visible: object.hasConsultant
                Layout.fillWidth: true
                Layout.preferredHeight: 40
                Style.class: "consultantDelete"
                textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                spacing: 16
                Tr { code: "positionConsultantDelete"; defaultMessage: "Удалить консультанта" }
                onClicked: consultantDelete(position)
            }
            Item {
                visible: object.hasConsultant
                Layout.preferredHeight: 24
            }
            RowLayout {
                Layout.alignment: Qt.AlignHCenter
                spacing: 24
                BasicButton {
                    Layout.preferredWidth: 72
                    Layout.preferredHeight: 56
                    Style.class: "positionQuant"
                    icon: Style.parameter("icon.minus")
                    enabled: Utils.weightGreater(object.bquant, 1.0) && object.unitFlag !== true
                    onClicked: changeQuantity(position, -1.0)
                }
                BasicButton {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 56
                    Style.class: "positionStorno"
                    textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                    Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                    onClicked: storno(position)
                }
                BasicButton {
                    Layout.preferredWidth: 72
                    Layout.preferredHeight: 56
                    Style.class: "positionQuant"
                    icon: Style.parameter("icon.plus")
                    enabled: object.unitFlag !== true
                    onClicked: changeQuantity(position, 1.0)
                }
            }
            Behavior on opacity { OpacityAnimator { duration: 500 } }
        }
    }
}
